/*
    Copyright (C) 2017 Vincent Delecroix

    This file is part of e-antic

    e-antic is free software: you can redistribute it and/or modify it under
    the terms of the GNU Lesser General Public License (LGPL) as published
    by the Free Software Foundation; either version 3.0 of the License, or
    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
*/


#include "../../e-antic/config.h"

#include "../../e-antic/renf_elem.h"

void renf_elem_randtest(renf_elem_t a, flint_rand_t state, mp_bitcnt_t bits, renf_t nf)
{
    /* TODO: this should be rational with higher probability */
    nf_elem_randtest(a->elem, state, bits, nf->nf);
    renf_elem_set_evaluation(a, nf, nf->prec);
}

